@@ -1,7 +1,7 @@
 package App::cryp::exchange;
 
-our $DATE = '2018-06-08'; # DATE
-our $VERSION = '0.005'; # VERSION
+our $DATE = '2018-06-11'; # DATE
+our $VERSION = '0.006'; # VERSION
 
 use 5.010001;
 use strict;
@@ -10,28 +10,6 @@ use Log::ger;
 
 our %SPEC;
 
-our $_complete_exchange = sub {
-    require Complete::Util;
-    require PERLANCAR::Module::List;
-
-    my %args = @_;
-
-    my $mods = PERLANCAR::Module::List::list_modules(
-        "App::cryp::Exchange::", {list_modules=>1});
-
-    my @safenames;
-    for (sort keys %$mods) {
-        s/.+:://;
-        s/_/-/g;
-        push @safenames, $_;
-    }
-
-    Complete::Util::complete_array_elem(
-        word  => $args{word},
-        array => \@safenames,
-    );
-};
-
 our %arg_detail = (
     detail => {
         schema => 'bool*',
@@ -45,10 +23,9 @@ our %arg_native = (
     },
 );
 
-our %arg_req0_exchange = (
-    exchange => {
-        schema => 'str*',
-        completion => $_complete_exchange,
+our %arg_req0_account = (
+    account => {
+        schema => 'cryptoexchange::account*',
         req => 1,
         pos => 0,
     },
@@ -74,42 +51,83 @@ our %arg_type = (
     },
 );
 
+our %arg_req2_type = (
+    type => {
+        schema => ['str*', in=>['buy','sell']],
+        req => 1,
+        pos => 2,
+        cmdline_aliases => {
+            buy  => {is_flag=>1, code=>sub {$_[0]{type}='buy' }, summary=>'Alias for --type=buy' },
+            sell => {is_flag=>1, code=>sub {$_[0]{type}='sell'}, summary=>'Alias for --type=sell'},
+        },
+    },
+);
+
+our %arg_req3_price = (
+    price => {
+        schema => ['float*', xmin=>0],
+        req => 1,
+        pos => 3,
+    },
+);
+
+our %args_size = (
+    base_size => {
+        summary => 'Order amount, denominated in base currency (first currency of the pair)',
+        schema => ['float*', xmin=>0],
+    },
+    quote_size => {
+        summary => 'Order amount, denominated in quote currency (second currency of the pair)',
+        schema => ['float*', xmin=>0],
+    },
+);
+
+our %arg_req3_order_id = (
+    order_id => {
+        schema => ['str*'],
+        req => 1,
+        pos => 3,
+    },
+);
+
 $SPEC{':package'} = {
     v => 1.1,
-    summary => 'Interact with cryptoexchanges',
+    summary => 'Interact with cryptoexchanges using a common interface',
 };
 
-sub _instantiate_exchange {
-    my ($r, $exchange, $account) = @_;
+sub _init {
+    my ($r) = @_;
 
-    my $mod = "App::cryp::Exchange::$exchange"; $mod =~ s/-/_/g;
-    (my $mod_pm = "$mod.pm") =~ s!::!/!g; require $mod_pm;
+  INSTANTIATE_EXCHANGE_CLIENT:
+    {
+        last unless $r->{args}{account};
+        my ($exchange, $account) = $r->{args}{account} =~ m!(.+)/(.+)!
+            or return [
+                400, "Invalid cryptoexchange account syntax ".
+                    "'$r->{args}{account}', please use EXCHANGE/ACCOUNT ".
+                    "format"];
+        my $mod = "App::cryp::Exchange::$exchange"; $mod =~ s/-/_/g;
+        (my $mod_pm = "$mod.pm") =~ s!::!/!g; require $mod_pm;
 
-    my $crypconf = $r->{_cryp};
+        my $hash = $r->{_cryp}{exchanges}{$exchange}{$account}
+            or return [404, "Unknown $exchange account $account"];
 
-    my %args = (
-    );
+        my %args = map { $_ => $hash->{$_} } grep {/^api_/} keys %$hash;
 
-    my $accounts = $crypconf->{exchanges}{$exchange} // {};
-    for my $a (sort keys %$accounts) {
-        if (!defined($account) || $account eq $a) {
-            for (grep {/^api_/} keys %{ $accounts->{$a} }) {
-                $args{$_} = $accounts->{$a}{$_};
-            }
-            last;
-        }
+        $r->{_stash}{exchange_client} = $mod->new(%args);
     }
-    $mod->new(%args);
+    [200];
 }
 
-$SPEC{list_exchanges} = {
+$SPEC{exchanges} = {
     v => 1.1,
     summary => 'List supported exchanges',
     args => {
         %arg_detail,
     },
+    tags => ['category:etc'],
 };
-sub list_exchanges {
+sub exchanges {
     require PERLANCAR::Module::List;
 
     my %args = @_;
@@ -136,15 +154,16 @@ sub list_exchanges {
     [200, "OK", \@res, $resmeta];
 }
 
-$SPEC{list_accounts} = {
+$SPEC{accounts} = {
     v => 1.1,
     summary => 'List exchange accounts',
     args => {
-        # XXX filter by exchnage (-I, -X)
+        # XXX filter by exchange (-I, -X)
         %arg_detail,
     },
+    tags => ['category:etc'],
 };
-sub list_accounts {
+sub accounts {
     my %args = @_;
 
     my $crypconf = $args{-cmdline_r}{_cryp};
@@ -173,21 +192,22 @@ sub list_accounts {
 
 }
 
-$SPEC{list_pairs} = {
+$SPEC{pairs} = {
     v => 1.1,
     summary => 'List pairs supported by exchange',
     args => {
-        %arg_req0_exchange,
+        %arg_req0_account,
         %arg_detail,
         %arg_native,
     },
 };
-sub list_pairs {
+sub pairs {
     my %args = @_;
 
     my $r = $args{-cmdline_r};
 
-    my $xchg = _instantiate_exchange($r, $args{exchange});
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
 
     $xchg->list_pairs(
         detail => $args{detail},
@@ -195,23 +215,24 @@ sub list_pairs {
     );
 }
 
-$SPEC{get_order_book} = {
+$SPEC{orderbook} = {
     v => 1.1,
     summary => 'Get order book on an exchange',
     args => {
-        %arg_req0_exchange,
+        %arg_req0_account,
         %arg_req1_pair,
         %arg_type,
     },
 };
-sub get_order_book {
+sub orderbook {
     my %args = @_;
 
     my $r = $args{-cmdline_r};
 
-    my $xchg = _instantiate_exchange($r, $args{exchange});
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
 
-    my $res = $xchg->get_order_book(
+    $res = $xchg->get_order_book(
         pair => $args{pair},
     );
     return $res unless $res->[0] == 200;
@@ -243,25 +264,93 @@ sub get_order_book {
     [200, "OK", \@rows];
 }
 
-$SPEC{list_balances} = {
+$SPEC{balance} = {
     v => 1.1,
-    summary => 'List account balances',
+    summary => 'List account balance',
     args => {
-        %arg_req0_exchange,
+        %arg_req0_account,
     },
 };
-sub list_balances {
+sub balance {
     my %args = @_;
 
     my $r = $args{-cmdline_r};
 
-    my $xchg = _instantiate_exchange($r, $args{exchange});
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
 
     $xchg->list_balances;
 }
 
+$SPEC{create_limit_order} = {
+    v => 1.1,
+    summary => 'Create a limit order',
+    args => {
+        %arg_req0_account,
+        %arg_req1_pair,
+        %arg_req2_type,
+        %arg_req3_price,
+        %args_size,
+    },
+    args_rels => {
+        req_one => [qw/base_size quote_size/],
+    },
+};
+sub create_limit_order {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->create_limit_order(%args);
+}
+
+$SPEC{get_order} = {
+    v => 1.1,
+    summary => 'Get information about an order',
+    args => {
+        %arg_req0_account,
+        %arg_req1_pair,
+        %arg_req2_type,
+        %arg_req3_order_id,
+    },
+};
+sub get_order {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->get_order(%args);
+}
+
+$SPEC{cancel_order} = {
+    v => 1.1,
+    summary => 'Cancel an order',
+    args => {
+        %arg_req0_account,
+        %arg_req1_pair,
+        %arg_req2_type,
+        %arg_req3_order_id,
+    },
+};
+sub cancel_order {
+    my %args = @_;
+
+    my $r = $args{-cmdline_r};
+
+    my $res = _init($r); return $res unless $res->[0] == 200;
+    my $xchg = $r->{_stash}{exchange_client};
+
+    $xchg->cancel_order(%args);
+}
+
 1;
-# ABSTRACT: Interact with cryptoexchanges
+# ABSTRACT: Interact with cryptoexchanges using a common interface
 
 __END__
 
@@ -271,11 +360,11 @@ __END__
 
 =head1 NAME
 
-App::cryp::exchange - Interact with cryptoexchanges
+App::cryp::exchange - Interact with cryptoexchanges using a common interface
 
 =head1 VERSION
 
-This document describes version 0.005 of App::cryp::exchange (from Perl distribution App-cryp-exchange), released on 2018-06-08.
+This document describes version 0.006 of App::cryp::exchange (from Perl distribution App-cryp-exchange), released on 2018-06-11.
 
 =head1 SYNOPSIS
 
@@ -284,45 +373,11 @@ Please see included script L<cryp-exchange>.
 =head1 FUNCTIONS
 
 
-=head2 get_order_book
+=head2 accounts
 
 Usage:
 
- get_order_book(%args) -> [status, msg, result, meta]
-
-Get order book on an exchange.
-
-This function is not exported.
-
-Arguments ('*' denotes required arguments):
-
-=over 4
-
-=item * B<exchange>* => I<str>
-
-=item * B<pair>* => I<str>
-
-=item * B<type> => I<str>
-
-=back
-
-Returns an enveloped result (an array).
-
-First element (status) is an integer containing HTTP status code
-(200 means OK, 4xx caller error, 5xx function error). Second element
-(msg) is a string containing error message, or 'OK' if status is
-200. Third element (result) is optional, the actual result. Fourth
-element (meta) is called result metadata and is optional, a hash
-that contains extra information.
-
-Return value:  (any)
-
-
-=head2 list_accounts
-
-Usage:
-
- list_accounts(%args) -> [status, msg, result, meta]
+ accounts(%args) -> [status, msg, result, meta]
 
 List exchange accounts.
 
@@ -348,13 +403,13 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 list_balances
+=head2 balance
 
 Usage:
 
- list_balances(%args) -> [status, msg, result, meta]
+ balance(%args) -> [status, msg, result, meta]
 
-List account balances.
+List account balance.
 
 This function is not exported.
 
@@ -362,7 +417,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<exchange>* => I<str>
+=item * B<account>* => I<cryptoexchange::account>
 
 =back
 
@@ -378,11 +433,91 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 list_exchanges
+=head2 cancel_order
 
 Usage:
 
- list_exchanges(%args) -> [status, msg, result, meta]
+ cancel_order(%args) -> [status, msg, result, meta]
+
+Cancel an order.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<account>* => I<cryptoexchange::account>
+
+=item * B<order_id>* => I<str>
+
+=item * B<pair>* => I<str>
+
+=item * B<type>* => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 create_limit_order
+
+Usage:
+
+ create_limit_order(%args) -> [status, msg, result, meta]
+
+Create a limit order.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<account>* => I<cryptoexchange::account>
+
+=item * B<base_size> => I<float>
+
+Order amount, denominated in base currency (first currency of the pair).
+
+=item * B<pair>* => I<str>
+
+=item * B<price>* => I<float>
+
+=item * B<quote_size> => I<float>
+
+Order amount, denominated in quote currency (second currency of the pair).
+
+=item * B<type>* => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 exchanges
+
+Usage:
+
+ exchanges(%args) -> [status, msg, result, meta]
 
 List supported exchanges.
 
@@ -408,11 +543,81 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 list_pairs
+=head2 get_order
 
 Usage:
 
- list_pairs(%args) -> [status, msg, result, meta]
+ get_order(%args) -> [status, msg, result, meta]
+
+Get information about an order.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<account>* => I<cryptoexchange::account>
+
+=item * B<order_id>* => I<str>
+
+=item * B<pair>* => I<str>
+
+=item * B<type>* => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 orderbook
+
+Usage:
+
+ orderbook(%args) -> [status, msg, result, meta]
+
+Get order book on an exchange.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<account>* => I<cryptoexchange::account>
+
+=item * B<pair>* => I<str>
+
+=item * B<type> => I<str>
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 pairs
+
+Usage:
+
+ pairs(%args) -> [status, msg, result, meta]
 
 List pairs supported by exchange.
 
@@ -422,9 +627,9 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<detail> => I<bool>
+=item * B<account>* => I<cryptoexchange::account>
 
-=item * B<exchange>* => I<str>
+=item * B<detail> => I<bool>
 
 =item * B<native> => I<bool>
 
